/**
 * Licensed Materials - Property of IBM
 * Rational ClearCase
 * (C) Copyright IBM Corp. 2012, 2014.  All Rights Reserved
 * US Government Users Restricted Rights -
 * Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has been
 * deposited with the U.S. Copyright Office.
 */
package com.ibm.rational.team.client.ui.external.extensions.interfaces;

import java.util.List;
import java.util.Properties;

import org.eclipse.swt.widgets.Shell;

/**
 * Implementors of this 'Task Provider' interface will activate support for the auto-creation of a
 * UCM activity for ClearCase operations running in the context of user selected 'Task'. When a
 * 'Task Provider' plugin is installed into the same Eclipse shell as the ClearTeam Explorer pluign
 * (CTE), CTE users will work primarily with 'Task's' as opposed to UCM activities. Although UCM
 * activities are still present and required by UCM ClearCase, CTE will manage their creation
 * automatically. The integration of 'Tasks' and UCM activities will help CTE users to think in
 * terms of 'Tasks' (e.g. a defect or enhancement), instead of UCM activities. For example, a CTE
 * user will be prompted to create or select a 'Task' to work on instead of a UCM activity. The
 * 'Task' is automatically linked to a UCM activity when necessary.
 * <p>
 * Example usage scenario:
 * <ul>
 * <li>A CTE user is working in a UCM context</li>
 * <li>The user creates a new 'Task' from CTE</li>
 * <li>The new 'Task' is created in the task provider's repository</li>
 * <li>CTE automatically create a new UCM activity
 * <li>
 * <li>CTE automatically links the new UCM activity to the new 'Task'</li>
 * <li>
 * </ul>
 * <p>
 * A 'Task' is a provider specific record that represents an item of work. For example, a 'work
 * item' in Rational Team Concert (RTC).
 * <p>
 * Current known implementors: RTC 4.0 or later<br>
 * Consumers: ClearTeam Explorer 8.0.0.2 or later
 * @param <IWorkbenchWindow>
 */
public interface ICCTaskAssociationProvider  {
    
    /**
     * Property keys that describe this Task Provider
     *
     */
    enum ProviderInfoPropertyKey {
        /**
         * The provider's unique identifier
         */
        PROVIDER_ID,
        /**
         * The provider version
         */
        PROVIDER_VERSION
    };
    
    /**
     * 
     * Property keys used to describe a ClearCase resource linked to a Task
     *
     */
    public static enum CCResourcePropertyKey {
        /**
         * The ClearCase resource universal resource identifier.
         */
        URI,
        
       /**
        * The ClearCase resource display string, e.g. activity:ID@/vob 
        */
        
        DISPLAY_STRING
    };
    
    /**
     * Property keys used to describe a 'Task' 
     *
     */
    public static enum TaskPropertyKey {
        /**
         * The task's universal resource identifier. This URI uniquely identifies a task across repositories.
         */
        URI,
        
        /**
         * The task id (may equal the URI). This ID uniquely identifies a task within a single repository.
         */
        ID,
        
        /**
         * the tasks headline
         */
        HEADLINE,
        
        /**
         * the task icon URL
         */
        ICON_URL        
    }
    
    /**
     * Property key that represents a context message. 
     *
     */
    public static enum MessagePropertyKey { 
        /**
         * A context message. A message may be passed via {@code Properties} to createTask or
         * selectTask to describe the context for the action.
         */
        CONTEXT_MESSAGE
    }

    /**
     * Retuns basic information about this task provider.
     * @return {@code Properties} representing the provider information
     * <p>
     * Properties must contain:
     * <ul>
     * <li>{@code ProviderInfoPropertyKey.PROVIDER_ID</li>
     * <li>{@code ProviderInfoPropertyKey.PROVIDER_VERSION</li>
     * </ul>
     */
    public Properties getProviderInfo();

    /**
     * Initialize the context for this provider. A provider's context is a set of provider specific
     * properties that represent a specific repository.
     * <p>
     * For example:<br>
     * Rational Team Concert describes its context in terms of a repository uri and a project area.
     * <p>
     * CTE may obtain this context via {@getContextByTask} and pass it to
     * {@code selectTask(...)} and/or {@code createTask(...)} to ensure a task is created or
     * selected in the repository specified by the context.
     * 
     * @param contextString
     *            If this task provider is integrated with the CCRC WAN Server, contextString
     *            contains the raw pvar string provided by the ClearCase administrator for this Task
     *            Provider. If this task provider is not integrated with the CCRC WAN Server,
     *            contextString is null
     * @return The context inferred from the pvar value
     * @throws Exception
     *             An error occurred
     */
    public Properties initContext(String contextString)
    throws Exception;

    /**
     * Return the context used by the the specified task. The context is a subset of key value
     * pairs contained within the specified task's {@code Properties}. The subset must equal the set
     * of properties returned by initContext.
     * 
     * @param task
     *        The task used to infer context
     * @return The context used by the specified task
     * @throws Exception
     *         An error occurred
     * @see #initContext(String)
     */
    public Properties getContextByTask(Properties task)
    throws Exception;
    
    /**
     * An object representation of the Task query. This object is Task Provider specific. It
     * is used as input to {@code hasNextTasksPage(...)} and {@code getNextTaskPage(...)}. It
     * is only understood by the Task Provider.
     * 
     * @param context
     *        The context for this provider as returned by {@code initContext(String)}
     * @param itemsPerPage
     *        The number of items to be displayed per page or -1 to display all Tasks in one page
     * @return A Task Provider specific representation of the Task query
     * @throws Exception
     *         An error occurred
     */
    public Object getTasksQuery(Properties context, int itemsPerPage) 
    throws Exception;
    
    /**
     * Returns whether the query returned by
     * {@code getTasksQuery(Properties context, int itemsPerPage} has a page of
     * task item results available.
     * 
     * @param taskQuery
     *            The task query as returned by {@code getTaskQuery(...)}
     * @return true, if a page of task items is available, false, if there are no more pages contain task items.
     * @throws Exception
     *             An error occurred
     * @see ICCTaskAssociationProvider#getTasksQuery(Properties, int)
     */
    public boolean hasNextTasksPage(Object taskQuery)
    throws Exception;
    
    /**
     * Returns the list of task items that are contained in the next tasks page. Each task item
     * contained in the list is represented as a {@code Properties} object
     * 
     * @param queryResult
     *            The task query as returned by {@code getTaskQuery(...)}
     * @return The list of task items, each represented as {@code Properties}
     *         <p>
     *         Each task's properties will include (at least):
     *         <ul>
     *         <li>{@code TaskPropertyKey.URI}</li>
     *         <li>{@code TaskPropertyKey.HEADLINE}</li>
     *         </ul>
     *         In addition, any Task Provider specific context information must be present.
     * @throws Exception
     *             An error occurred
     * @see ICCTaskAssociationProvider#getTasksQuery(Properties, int)
     */
    public List<Properties> getNextTasksPage(Object taskQuery)
    throws Exception;
    
    /**
     * Allows a user to create a new "task" (e.g. "work item" in RTC). Invokes any UI necessary to
     * enable a user to create a new task.
     * 
     * @param properties
     *        {@code Properties} used to create a new task. May be null.
     *        <p>
     *        <u>Optional</u> properties (not limited to):
     *        <ul>
     *        <li>{@code TaskPropertyKey.HEADLINE} - override the default task headline</li>
     *        <li>{@code MessagePropertyKey.MESSAGE_CONTEXT} - the message to display in the create dialog</li>
     *        </ul>
     *        In addition, if any Task Provider specific context information is present, the default
     *        context will be overridden.
     * @return {@code Properties} describing the new task or {@code null} if the user canceled task
     *         creation.
     *         <p>
     *         These properties must include (at least):
     *         <ul>
     *         <li>{@code TaskPropertyKey.URI}</li>
     *         <li>{@code TaskPropertyKey.HEADLINE}</li>
     *         </ul>
     *         In addition, any Task Provider specific context information must be present.
     * @exception Exception
     *            An error occurred
     */
    public Properties createTask(Shell shell, Properties properties) 
    throws Exception;
    
    /**
     * Allows a user to select a "task" (e.g. "work item" in RTC). Invokes any UI necessary to
     * enable a user to choose task.
     * 
     * @param properties
     *        {@code Properties} used to select task(s). May be null.
     *        <p>
     *        <u>Optional</u> properties (not limited to):
     *        <ul>
     *        <li>{@code TaskPropertyKey.HEADLINE} - override the default task headline</li>
     *        <li>{@code MessagePropertyKey.MESSAGE_CONTEXT} - the message to display in the create dialog</li>
     *        </ul>
     *        In addition, if any Task Provider specific context information is present, the default
     *        context will be overridden.
     * @param allowNewTask
     *        Allows user to create a new "task"
     * @return The {@code Properties} describing the selected task.
     *         <p>
     *         The properties must include (at least):
     *         <ul>
     *         <li>{@code TaskPropertyKey.URI}</li>
     *         <li>{@code TaskPropertyKey.HEADLINE}</li>
     *         </ul>
     *         In addition, any Task Provider specific context information must be present.
     *         </p>
     *         null is returned if there is no selection.
     * 
     * @exception Exception
     *            An error occurred
     */
    public Properties selectTask(Shell shell, Properties properties, boolean allowNewTask)
    throws Exception;
    
    /**
     * Associates the given ClearCase object (e.g. UCM Activity or ClearCase Version) with the list
     * of "tasks" (e.g. "work items" in RTC)
     * 
     * @param ccResourceId
     *        A string representation of a ClearCase resource e.g. {@code GIObject.toString()} to
     *        associate tasks
     * @param taskUris
     *        a list of {@code Properties}, where each item contains the the URI of a "task" to
     *        associate with the ccResourceId.
     *        <p>
     *        Each {@code Properties} object <b>must</b> contain a valid value for the
     *        {@code TaskPropertyKey.URI} key
     * @exception Exception
     *            An error occurred
     */
   public void associateTasks(String ccResourceId, List<Properties> taskUris)
   throws Exception;
   
    /**
     * Dissociates all "tasks" from the given ClearCase object (e.g. UCM Activity or ClearCase
     * Version)
     * 
     * @param ccResourceId
     *            A string representation of a ClearCase resource e.g. {@code GIObject.toString()}
     * @param repository
     *            {@code Properties} object containing the repository information to dissociate.
     *            <p>
     *            To specify a repository, the {@code Properties} object must contain
     *            {@code RepositoryPropertyKey.REPOSITORY_URI}<br>
     *            If repository does not contain {@code RepositoryPropertyKey.REPOSITORY_URI}, links
     *            from all repositories dissociated
     * @exception Exception
     *                An error occurred
     */
   public void dissociateTasks(String ccResourceId, Properties repository)
   throws Exception;
   
    /**
     * Get the list of "tasks" (e.g. "work items" in RTC) associated with the given ClearCase
     * resource (e.g. UCM Activity or ClearCase Version)
     * 
     * @param ccResourceId
     *            A string representation of a ClearCase resource e.g. {@code GIObject.toString()}
     *            whose task association we will fetch
     * @param repository
     *            {@code Properties} object containing the repository to query.
     *            <p>
     *            To specify a repository, the {@code Properties} object must contain
     *            {@code RepositoryPropertyKey.REPOSITORY_URI}<br>
     *            If repository does not contain {@code RepositoryPropertyKey.REPOSITORY_URI},
     *            association from all repositories are returned
     * @return A list of {@code Properties} describing each associated task.
     *         <p>
     *         The properties will include (at least):
     *         <ul>
     *         <li>{@code TaskPropertyKey.URI}</li>
     *         <li>{@code TaskPropertyKey.HEADLINE}</li>
     *         </ul>
     *         An empty list is returned when no task associations exist for the cc resource
     *         </p>
     * @exception Exception
     *                An error occurred
     */
   public List<Properties> getTaskAssociations(String ccResourceId, Properties repository)
   throws Exception;

    /**
     * Get the list of ClearCase Resource identifiers (e.g. UCM activity URIs or ClearCase version
     * URIs) associated with the specified task.
     * 
     * @param task
     *            The ClearCase resource represented as properties
     *            <p>
     *            Required properties include:
     *            <ul>
     *            <li>{@code TaskPropertyKey.URI}</li>
     *            </ul>
     * @param isUCM
     *            if true, return only UCM activity URIs, if false, return only Base CC version URIs
     * @return The list of cc resource associations as strings
     * @throws Exception
     *             An error occurred
     */
   @Deprecated
   public List<String> getCcResourceAssociations(Properties task, boolean isUCM)
   throws Exception;
   
   /**
    * Get the list of Properties of ClearCase resources associated with the specified task.
    * 
    * @param task
    *            The Task represented as properties
    *            <p>
    *            Required properties include:
    *            <ul>
    *            <li>{@code TaskPropertyKey.URI}</li>
    *            </ul>
    * @param isUCM
    *            if true, return only UCM activity URIs, if false, return only Base CC version URIs
    * @return The list of cc resource associations as Properties
    *           <p>
    *           Required properties include:
    *           <ul>
    *           <li>{@code CCResourcePropertyKey.URI}</li>
    *           <li>{@code CCResourcePropertyKey.DISPLAY_STRING}</li>
    *           </ul>
    * @throws Exception
    *             An error occurred
    */
   public List<Properties> getAssociatedCcResourceProperties(Properties task, boolean isUCM)
   throws Exception;
   
    /**
     * Get the current task. The current task is a single 'Task' that the 'Task Provider'
     * acknowledges as the current working task.
     * 
     * @return Properties describing the current task or null, if there is no current task
     *         <p>
     *         The properties will include (at least):
     *         <ul>
     *         <li>{@code TaskPropertyKey.URI}</li>
     *         <li>{@code TaskPropertyKey.HEADLINE}</li>
     *         </ul>
     *         </p>
     * @throws Exception
     *             An error occured
     */
   public Properties getCurrentTask()
   throws Exception;
   
    /**
     * Set the current task. The current task is a single 'Task' that the 'Task Provider'
     * acknowledges as the current working task.
     * 
     * @param task
     *            Properties describing the current task
     *            <p>
     *            The properties must include (at least):
     *            <ul>
     *            <li>{@code TaskPropertyKey.URI}</li>
     *            </ul>
     * @throws Exception
     *             An error occurred
     */
    public void setCurrentTask(Properties task)
    throws Exception;
  
    /**
     * Returns a list of tasks as {@code Properties}. For example, this represents a list of tasks
     * assigned to the CTE user. It is recommended that this list be limited to work that can be
     * completed by working on a particular development stream.
     * <p>
     * This list will display under the ClearTeam Explorer's 'My Activities' node,the common dialog
     * activity pickers, etc.
     * 
     * @param query
     * @return a list of tasks as {@code Properties}
     */
    public List<Properties> getTaskList(Properties query);
    
    
    /**
     * Open tasks in the provider's UI
     * 
     * @param tasks
     *             A list of {@code Properties}, where each item contains the the URI of a "task" to open
     *         
     * @param window
     *             A Workbench window to open the task in
     *             
     * @throws Exception
     *             An error occurred      
     */
    public void openTasks(List<Properties> tasks, Object window)
    throws Exception;

    /**
     * This callback enables CTE to respond when the current task is set or cleared. The current
     * task is a single 'Task' that the 'Task Provider' acknowledges as the current working task.
     * The 'Task Provider' plugin should call this callback's methods when the 'Task Provider's'
     * current task has changed or been cleared. If this 'Task Provider' has no concept of a current
     * task, this 'Task Provider' may ignore this callback completely.
     * 
     * An implementation provided by the ClearTeam Explorer (CTE). This callback is by CTE on
     * startup.
     * 
     * @param callback
     */
    public void cteNotificationCallback(ICCTaskAssocationCallback callback);
  
    public interface ICCTaskAssocationCallback {
        /**
         * Notify CTE that the current task was set
         * 
         * @param task
         */
        void currentTaskSetNotify(Properties task) throws Exception;

        /**
         * Notify CTE that the current task was cleared.
         * 
         * @param task
         */
        void currentTaskClearNotify(Properties task);
    }
}
